getmapping<-function(projinfo,dom) {

  # get coordinates from WRF
  if (dom<10) filename <- paste(wrfinput,"/wrfinput_d0",dom,sep="")
  if (dom>9 ) filename <- paste(wrfinput,"/wrfinput_d",dom,sep="")

  # load wrf grid
  library(RNetCDF)
  nc       <- open.nc(filename)
  lat.wrf  <- t(var.get.nc(nc,"XLAT_V"))
  lon.wrf  <- t(var.get.nc(nc,"XLONG_U"))  
  landmask <- t(var.get.nc(nc, "LANDMASK"))
  close.nc(nc)

  # get mapping
  if (!file.exists(paste(datp,"/.RDatamapping.LCC.",dom,sep=""))) {

   # aggregate map and indices
   synmap <- getr(paste("vegetationmapCutted.",projinfo$year,".",dom,sep=""),datp)
   lat    <- as.numeric(dimnames(synmap)[[1]])
   lat    <- lat-diff(lat)[1] #from upper left to lower left (spacing equivalent)
   lon    <- as.numeric(dimnames(synmap)[[2]])
   lon.in <- rep(1,length(lat))%o%lon
   lat.in <- lat%o%rep(1,length(lon))   
    
   mapping <- c(NA,NA,NA,NA,NA) # ini, inj, outi, outj, sum(sel)

   for (i in 1:(dim(lat.wrf)[1]-1)) {
    print(date())
    
    # get smaller grid to speed up inner loop
    lat.range <- range(lat.wrf[i:(i+1),])
    ind.range <- which(lat.in>=lat.range[1]&lat.in<=lat.range[2],arr.ind=T)
    if (length(ind.range)==0) break
    row.range <- range(ind.range[,1])
    lat.small <- lat.in[row.range[1]:row.range[2],]
    lon.small <- lon.in[row.range[1]:row.range[2],]
    offset    <- row.range[1]-1  
    
    for (j in 1:(dim(lon.wrf)[2]-1)) {
     #print(paste(j,date()))
     sel <- lat.small>=lat.wrf[i,j] & lat.small<lat.wrf[i+1,j] & lon.small>=lon.wrf[i,j] & lon.small<lon.wrf[i,j+1]
     if (sum(sel)>0) {
      selection <- which(sel==TRUE,arr.ind=T)
      selection <- cbind(selection[,1]+offset,selection[,2])
      mapping <- rbind(mapping,cbind(selection,i,j,sum(sel)))
     }
    }
    print(paste(i/(dim(lat.wrf)[1]-1)*100,"%"))    
   }
   mapping <- mapping[2:(length(mapping)/5),]
   assignr(paste("mapping.LCC.",dom,sep=""), mapping, datp)
  }  
}


# do getmapping on the cluster in parallel
getmappingParallel<-function(projinfo,dom) {

  # get mapping
  if (file.exists(paste(datp,"/.RDatamapping.LCC.",dom,sep=""))) return(NULL)

  # call script: needs vari, dom, notsteps
  system(paste(scriptpath,"/submit_getmapping.sh ",scriptpath," ",dom," ",projinfo$ny[dom],sep=""))

  mapping <- NULL # ini, inj, outi, outj, sum(sel)

  # load and combine all sub mappings
  for (file in dir(datp,pattern=paste(".RDatalsf_mapping.LCC.",dom,"*",sep=""),all.files=TRUE)) {
   mapping <- rbind(mapping, getr(strsplit(file,".RData")[[1]][2],datp))
  }
  
  assignr(paste("mapping.LCC.",dom,sep=""), mapping, datp)
}


aggregateMapLCC<-function(dom,projinfo,nveg,notsteps) {
		       
  if (file.exists(paste(datp,"/.RDatamapAggregated.",projinfo$year,".",dom,sep=""))) return(NULL)

  # get coordinates from WRF
  if (dom<10) filename <- paste(wrfinput,"/wrfinput_d0",dom,sep="")
  if (dom>9 ) filename <- paste(wrfinput,"/wrfinput_d",dom,sep="")

  library(RNetCDF)
  nc    <- open.nc(filename)
  landmask <- t(var.get.nc(nc, "LANDMASK"))
  close.nc(nc)

  # aggregate map
  synmap <- getr(paste("vegetationmapCutted.",projinfo$year,".",dom,sep=""),datp)
  # get mapping 
  mapping <- getr(paste("mapping.LCC.",dom,sep=""), datp)

  vegmap <- array(0, dim=c(dim(landmask), nveg))

  for (v in 1:nveg) {
   inmat <- synmap*(synmap==v)*1
   tmp   <- landmask*0
   for (ind in 1:dim(mapping)[1]) { 
    tmp[mapping[ind,3],mapping[ind,4]]  <- sum(c(tmp[mapping[ind,3],mapping[ind,4]],inmat[mapping[ind,1],mapping[ind,2]]))  
   }
   vegmap[,,v] <- tmp
  }

  #from counts to rel. coverage
  sumsyn <- array(0,dim=dim(vegmap)[1:2])
  for(v in 1:nveg) sumsyn <- sumsyn+vegmap[,,v]
  for(v in 1:nveg) vegmap[,,v] <- vegmap[,,v]/sumsyn

  # save
  assignr(paste("mapAggregated.",projinfo$year,".",dom,sep=""),vegmap,datp)   
}

aggregateIndicesLCC <- function(vari,projinfo,dom,notsteps,nveg) {

  # for debugging:		         
  # dom <- 1; vari <- "evi"; v <- vegclass <- 6; notsteps <- 34; tstep <- 20
  
  if (file.exists(paste(datp,"/.RData",vari,".indicesAggregated.",projinfo$year,".",dom,sep="")))
  {
    print(paste(datp,"/.RData",vari,".indicesAggregated.",projinfo$year,".",dom,
    " already created. Skipping.",sep=""))
    return(NULL)
  }

  # get WRF grid
  if (dom<10) filename <- paste(wrfinput,"/wrfinput_d0",dom,sep="")
  if (dom>9 ) filename <- paste(wrfinput,"/wrfinput_d",dom,sep="")

  library(RNetCDF)
  nc    <- open.nc(filename)
  landmask <- t(var.get.nc(nc, "LANDMASK"))
  close.nc(nc)

  indicesAggregated <- array(0,dim=c(dim(landmask),notsteps,nveg))

  mapping <- getr(paste("mapping.LCC.",dom,sep=""), datp)
  for (tstep in 1:notsteps) {
   print(date())  
   indicesClassed <- getr(paste(vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""),datp)
   for (v in 1:nveg) {      
    inmat <- indicesClassed[,,v]
    tmp   <- landmask*0
    # sums is the sum of aggregated pixel per grid cell in the output matrix
    sums  <- landmask*0
    for (ind in 1:dim(mapping)[1]) {
     val <- indicesClassed[mapping[ind,1],mapping[ind,2],v]
     tmp[mapping[ind,3],mapping[ind,4]]  <- sum(c(tmp[mapping[ind,3],mapping[ind,4]],val),na.rm=T)
     sums[mapping[ind,3],mapping[ind,4]] <- sum(c(sums[mapping[ind,3],mapping[ind,4]],val/val),na.rm=T)   
    }
    # calculate average
    indicesAggregated[,,tstep,v] <- tmp/sums   
   } 
   print(paste(tstep/notsteps*100,"% complete"))
  }


  # save result
  assignr(paste(vari,".indicesAggregated.",projinfo$year,".",dom,sep=""),indicesAggregated,datp)  
  
  return(NULL)
}

# use cluster to regrid LCC fields
aggregateIndicesLCCParallel <- function(vari,projinfo,dom,notsteps,nveg) {

  # for debugging:		         
  # dom <- 1; vari <- "evi"; v <- vegclass <- 6; notsteps <- 34; tstep <- 20
  
  if (file.exists(paste(datp,"/.RData",vari,".indicesAggregated.",projinfo$year,".",dom,sep="")))
  {
    print(paste(datp,"/.RData",vari,".indicesAggregated.",projinfo$year,".",dom,
    " already created. Skipping.",sep=""))
    return(NULL)
  }

  # call script: needs vari, dom, notsteps
  system(paste(scriptpath,"/submit_regrid.sh ",scriptpath," ",vari," ",dom," ",notsteps,sep=""))

  ## combine results ##

  # get WRF grid
  if (dom<10) filename <- paste(wrfinput,"/wrfinput_d0",dom,sep="")
  if (dom>9 ) filename <- paste(wrfinput,"/wrfinput_d",dom,sep="")

  library(RNetCDF)
  nc    <- open.nc(filename)
  landmask <- t(var.get.nc(nc, "LANDMASK"))
  close.nc(nc)

  indicesAggregated <- array(0,dim=c(dim(landmask),notsteps,nveg))

  for (tstep in 1:notsteps) {
   print(tstep)
   curInd <- getr(paste(vari,".indicesAggregated.",projinfo$year,".",dom,"_tstep",tstep,sep=""),datp) 
   indicesAggregated[,,tstep,] <- curInd
  }
  
  # save result
  assignr(paste(vari,".indicesAggregated.",projinfo$year,".",dom,sep=""),indicesAggregated,datp)  
  
  return(NULL)
}



testagg <- function(dom,vari,tstep,v) {
 # test aggregation
 # get coordinates from WRF
 if (dom<10) filename <- paste(wrfinput,"/wrfinput_d0",dom,sep="")
 if (dom>9 ) filename <- paste(wrfinput,"/wrfinput_d",dom,sep="")

 # load wrf grid
 library(RNetCDF)
 nc       <- open.nc(filename)
 lat.wrf  <- t(var.get.nc(nc,"XLAT_V"))
 lon.wrf  <- t(var.get.nc(nc,"XLONG_U"))  
 landmask <- t(var.get.nc(nc, "LANDMASK"))
 close.nc(nc)

 # aggregate map and indices
 synmap <- getr(paste("vegetationmapCutted.",projinfo$year,".",dom,sep=""),datp)
 lat    <- as.numeric(dimnames(synmap)[[1]])
 lat    <- lat-diff(lat)[1] #from upper left to lower left (spacing equivalent)
 lon    <- as.numeric(dimnames(synmap)[[2]])
 lon.in <- rep(1,length(lat))%o%lon
 lat.in <- lat%o%rep(1,length(lon))    
 
 inmat  <- getr(paste(vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""),datp)[,,v]
 outmat <- landmask*0
 
 for (i in 1:(dim(lat.wrf)[1]-1)) {
  print(date())
  for (j in 1:(dim(lon.wrf)[2]-1)) {
   sel <- lat.in>=lat.wrf[i,j] & lat.in<lat.wrf[i+1,j] & lon.in>=lon.wrf[i,j] & lon.in<lon.wrf[i,j+1]
   if (sum(sel)>0) {
    outmat[j,i] <- mean(inmat[sel],na.rm=T)
   }
  }
  print(paste(i/(dim(lat.wrf)[1]-1)*100,"%"))    
 }
 
 save(file="outmat.aggtest",outmat) 
}




doAggregateLCC<-function(vari="evi",dom=1,doyuse,nveg) {

  projinfo <- getr("projinfo",path=datp)
  notsteps <- length(doyuse)
  
  #get mapping
  if (do_parallel) {
   getmappingParallel(projinfo,dom)  
  } else {
   getmapping(projinfo,dom)
  }
  
  # aggregate map
  aggregateMapLCC(dom,projinfo,nveg,notsteps)
  
  # aggregate indices
  if (do_parallel) {
   aggregateIndicesLCCParallel(vari,projinfo,dom,notsteps,nveg)
  } else {
   aggregateIndicesLCC(vari,projinfo,dom,notsteps,nveg)
  }

  return(NULL)
}















# use external NCL routine to do the regridding
doAggInNCL <- function() {

  # get coordinates from WRF
  if (dom<10) filename <- paste(wrfinput,"/wrfinput_d0",dom,sep="")
  if (dom>9 ) filename <- paste(wrfinput,"/wrfinput_d",dom,sep="")

  # load wrf grid
  require(RNetCDF)
  nc   <- open.nc(filename)
  xlat <- t(var.get.nc(nc,"XLAT"))
  xlon <- t(var.get.nc(nc,"XLONG"))  
  close.nc(nc)
 
  # write WRF grid to ncdf
  nc <- create.nc(paste(mpath,"/wrfgrid.nc",sep=""))
  dim.def.nc(nc,"west_east"  , dimlength=dim(xlon)[1])
  dim.def.nc(nc,"south_north", dimlength=dim(xlat)[2])

  var.def.nc(nc, "xlat", "NC_FLOAT", c(0,1))
  att.put.nc(nc, "xlat", "units", "NC_CHAR", "degrees_north") 
  att.put.nc(nc, "xlat", "long_name", "NC_CHAR", "latitude coordinate") 
  att.put.nc(nc, "xlat", "standard_name", "NC_CHAR", "latitude") 

  var.def.nc(nc, "xlon", "NC_FLOAT", c(0,1))
  att.put.nc(nc, "xlon", "units", "NC_CHAR", "degrees_east") 
  att.put.nc(nc, "xlon", "long_name", "NC_CHAR", "longitude coordinate") 
  att.put.nc(nc, "xlon", "standard_name", "NC_CHAR", "longitude")
  
  var.put.nc(nc, "xlat", xlat)
  var.put.nc(nc, "xlon", xlon)
  close.nc(nc)
  
  # load synmap
  synmap <- getr(paste("vegetationmapCutted.",projinfo$year,".",dom,sep=""),datp)
  lat    <- projinfo$lat.syn
  lon    <- projinfo$lon.syn

  lat	 <- as.numeric(dimnames(synmap)[[1]])
  lat	 <- lat-diff(lat)[1] #from upper left to lower left (spacing equivalent)
  lon	 <- as.numeric(dimnames(synmap)[[2]])

  # write synmap to ncdf
  nc <- create.nc(paste(mpath,"/dataIn.nc",sep=""))
  dim.def.nc(nc,"lat", dimlength=length(lat))
  dim.def.nc(nc,"lon", dimlength=length(lon))
  dim.def.nc(nc,"veg", dimlength=8)

  var.def.nc(nc, "lat", "NC_FLOAT", 0)
  att.put.nc(nc, "lat", "units", "NC_CHAR", "degrees_north") 
  att.put.nc(nc, "lat", "long_name", "NC_CHAR", "latitude coordinate") 
  att.put.nc(nc, "lat", "standard_name", "NC_CHAR", "latitude") 

  var.def.nc(nc, "lon", "NC_FLOAT", 1)
  att.put.nc(nc, "lon", "units", "NC_CHAR", "degrees_east") 
  att.put.nc(nc, "lon", "long_name", "NC_CHAR", "longitude coordinate") 
  att.put.nc(nc, "lon", "standard_name", "NC_CHAR", "longitude")

  var.def.nc(nc, "dat", "NC_INT", c(1,0,2))
  att.put.nc(nc, "dat", "missing_value", "NC_INT", -99999)
  
  var.put.nc(nc, "lat", lat)
  var.put.nc(nc, "lon", lon)
  for (v in 1:8) {
   tmp <- synmap
   tmp[synmap!=v] <- NA
   var.put.nc(nc, "dat", t(tmp), start=c(1,1,v), count=c(length(lon),length(lat),1))
  }
  close.nc(nc)
  
  # free memory
  rm(tmp);rm(synmap);gc()
  
  # call ncl 
  system("/Net/Groups/BSY/people/rkretsch/preprocessing/ncl_ncarg-5.1.1.Linux_x86_64_gcc330/bin/ncl /Net/Groups/BSY/people/rkretsch/preprocessing/VPRM/VPRMpreproc_LCC/trunk/regrid.ncl")

  date()
  
  # 11:10 -
  
  # load synmap
  nc   <- open.nc("dataOut.nc")
  map2 <- var.get.nc(nc,"dat")
  close.nc(nc)
  
  # calculate relative coverage
  sumsyn <- array(0,dim=dim(vegmap)[1:2])
  for(v in 1:nveg) sumsyn <- sumsyn+vegmap[,,v]
  for(v in 1:nveg) vegmap[,,v] <- vegmap[,,v]/sumsyn  

  # write R object
  
  # remove nc files

}



