# script to regrid indices to WRF grid on the cluster


scriptpath <- getwd() # path of preprocessor
sourcepath <-paste(scriptpath,"/RSources/",sep="")

source(paste(scriptpath,"/RSources/monthdayyear.r",sep=""))

# read configuration
source(paste(scriptpath,"/config.r",sep=""))

ftdata   <- FALSE          # shall we connect to ftp and check for data?
projtype <- "LCC"          # type of projection LCC xor GEO
datp <- rdataout
mdy      <- month.day.year
attachn  <- function(what, pos = 2){if(!(paste("file:",what,sep="")%in%search()))attach(what,pos)}
go       <- graphics.off

# set synmap resolution
spacing <- 1/120

# set number of used vegetation classes
nveg <- 8 

# create working directories
if (!file.exists(mpath)) dir.create(mpath,recursive=TRUE)
if (!file.exists(rdatain)) dir.create(rdatain,recursive=TRUE)
if (!file.exists(rdataout)) dir.create(rdataout,recursive=TRUE)

# check for synmap
if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
  #stop(paste("Synmap ",rdatain,"/.RDatasynmap.",synmaptouse," not found. Use ",sourcepath,"/gridEurope.r to obtain synmap!",sep=""));
  print("SYNMAP not there. Prepare one...")
  source(paste(scriptpath,"/RSources/gridEurope.r",sep=""))
  if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
   stop("Could not prepare SYNMAP!")
  }
}

setwd(rdataout)

library("hdf5")

source(paste(scriptpath,"/RSources/assignr.r",sep=""))
source(paste(scriptpath,"/RSources/project.nice.r",sep=""))
source(paste(scriptpath,"/RSources/getminmax.r",sep=""))
source(paste(scriptpath,"/RSources/distance.r",sep=""))
source(paste(scriptpath,"/RSources/getr.r",sep=""))
source(paste(scriptpath,"/RSources/loess.r",sep=""))
source(paste(scriptpath,"/RSources/imagell.r",sep=""))
source(paste(scriptpath,"/RSources/na_replace.r",sep=""))
source(paste(scriptpath,"/RSources/combineTiles.r",sep=""))
source(paste(scriptpath,"/RSources/hdfread.r",sep=""))
source(paste(scriptpath,"/RSources/parseNamelist.r",sep=""))
source(paste(scriptpath,"/RSources/getModis.r",sep=""))
source(paste(scriptpath,"/RSources/extractAndReduce.r",sep=""))
source(paste(scriptpath,"/RSources/doResampling.r",sep=""))
source(paste(scriptpath,"/RSources/readIndicesIntoR.r",sep=""))
source(paste(scriptpath,"/RSources/cutToDomain.r",sep=""))
source(paste(scriptpath,"/RSources/doAggregation.r",sep=""))
source(paste(scriptpath,"/RSources/writeOutput.r",sep=""))

# load grid configuration
projinfo <- getr("projinfo",datp)

# set 8 day intervals to process
# to do: calculate from start and end date set in namelist!
doyuse <- doyall  <- seq(as.numeric(paste(projinfo$year,"009",sep="")),as.numeric(paste(projinfo$year,"361",sep="")),by=8)
#doyuse   <- seq(as.numeric(paste(projinfo$year,"041",sep="")),as.numeric(paste(projinfo$year,"305",sep="")),by=8)

# cd to working directory
setwd(mpath)


  
# get from command line
args <- commandArgs()
print(args)
vari  <- args[5]
dom   <- as.numeric(args[6])
tstep <- as.numeric(args[7])



# skip if already done
if (file.exists(paste(datp,"/.RData",vari,".indicesAggregated.",projinfo$year,".",dom,"_tstep",tstep,sep=""))) quit()




# get WRF grid
if (dom<10) filename <- paste(wrfinput,"/wrfinput_d0",dom,sep="")
if (dom>9 ) filename <- paste(wrfinput,"/wrfinput_d",dom,sep="")

library(RNetCDF)
nc    <- open.nc(filename)
landmask <- t(var.get.nc(nc, "LANDMASK"))
close.nc(nc)

indicesAggregated <- array(0,dim=c(dim(landmask),nveg))

mapping <- getr(paste("mapping.LCC.",dom,sep=""), datp)

indicesClassed <- getr(paste(vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""),datp)
for (v in 1:nveg) {
 print(date())
 print(v)
 inmat <- indicesClassed[,,v]
 tmp   <- landmask*0
 # sums is the sum of aggregated pixel per grid cell in the output matrix
 sums  <- landmask*0
 for (ind in 1:dim(mapping)[1]) {
  val <- indicesClassed[mapping[ind,1],mapping[ind,2],v]
  tmp[mapping[ind,3],mapping[ind,4]]  <- sum(c(tmp[mapping[ind,3],mapping[ind,4]],val),na.rm=T)
  sums[mapping[ind,3],mapping[ind,4]] <- sum(c(sums[mapping[ind,3],mapping[ind,4]],val/val),na.rm=T)   
 }
 # calculate average
 indicesAggregated[,,v] <- tmp/sums   
} 


# save result
assignr(paste(vari,".indicesAggregated.",projinfo$year,".",dom,"_tstep",tstep,sep=""),indicesAggregated,datp)  

